/*
 * (C) Copyright 2009 Digi International, Inc.
 *     Derived from imx51.c
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <nvram.h>
#include <i2c.h>

#include <asm/io.h>
#include <asm/arch/mx51.h>
#include <asm/arch/mx51_pins.h>
#include <asm/arch/iomux.h>
#include <asm/errno.h>
#include <imx_spi.h>
#include <asm/arch/imx_spi_pmic.h>
#include <asm/arch/gpio.h>
#include <asm/arch/mxc_nand.h>
#include <netdev.h>

#ifdef CONFIG_CMD_MMC
#include <mmc.h>
#include <fsl_esdhc.h>
#endif

#ifdef CONFIG_ARCH_MMU
#include <asm/mmu.h>
#include <asm/arch/mmu.h>
#endif

DECLARE_GLOBAL_DATA_PTR;

static u32 system_rev;
struct spi_slave *pmicslv = NULL;
unsigned long boot_verb = 0;

#define PMIC_SET_ALL_GPO_LOW_MSK	((3 << 10) | (3 << 8))

u32 get_board_rev(void)
{
	return system_rev;
}

static inline void setup_soc_rev(void)
{
	int reg;
	reg = __REG(ROM_SI_REV);
	switch (reg) {
	case 0x10:	system_rev = 0x51000 | CHIP_REV_2_5;	break;
	case 0x20:	system_rev = 0x51000 | CHIP_REV_3_0;	break;
	default:	system_rev = 0x51000 | CHIP_REV_2_5;	break;
	}
}

static inline void set_board_rev(int rev)
{
	system_rev |= (rev & 0xF) << 8;
}

inline int is_soc_rev(int rev)
{
	return (system_rev & 0xFF) - rev;
}

#ifdef CONFIG_ARCH_MMU
void board_mmu_init(void)
{
       unsigned long ttb_base = PHYS_SDRAM_1 + 0x4000;
       unsigned long i;

       /*
       * Set the TTB register
       */
       asm volatile ("mcr  p15,0,%0,c2,c0,0" : : "r"(ttb_base) /*:*/);

       /*
       * Set the Domain Access Control Register
       */
       i = ARM_ACCESS_DACR_DEFAULT;
       asm volatile ("mcr  p15,0,%0,c3,c0,0" : : "r"(i) /*:*/);

       /*
       * First clear all TT entries - ie Set them to Faulting
       */
       memset((void *)ttb_base, 0, ARM_FIRST_LEVEL_PAGE_TABLE_SIZE);
       /* Actual   Virtual  Size   Attributes          Function */
       /* Base     Base     MB     cached? buffered?  access permissions */
       /* xxx00000 xxx00000 */
       X_ARM_MMU_SECTION(0x000, 0x200, 0x1,
                       ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                       ARM_ACCESS_PERM_RW_RW); /* ROM */
       X_ARM_MMU_SECTION(0x1FF, 0x1FF, 0x001,
                       ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                       ARM_ACCESS_PERM_RW_RW); /* IRAM */
       X_ARM_MMU_SECTION(0x300, 0x300, 0x100,
                       ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                       ARM_ACCESS_PERM_RW_RW); /* GPU */
       X_ARM_MMU_SECTION(0x400, 0x400, 0x200,
                       ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                       ARM_ACCESS_PERM_RW_RW); /* IPUv3D */
       X_ARM_MMU_SECTION(0x600, 0x600, 0x300,
                       ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                       ARM_ACCESS_PERM_RW_RW); /* periperals */
       X_ARM_MMU_SECTION(0x900, 0x000, 0x1FF,
                       ARM_CACHEABLE, ARM_BUFFERABLE,
                       ARM_ACCESS_PERM_RW_RW); /* SDRAM */
       X_ARM_MMU_SECTION(0x900, 0x900, 0x200,
                       ARM_CACHEABLE, ARM_BUFFERABLE,
                       ARM_ACCESS_PERM_RW_RW); /* SDRAM */
       X_ARM_MMU_SECTION(0x900, 0xE00, 0x200,
                       ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                       ARM_ACCESS_PERM_RW_RW); /* SDRAM 0:128M*/
       X_ARM_MMU_SECTION(0xB80, 0xB80, 0x10,
                       ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                       ARM_ACCESS_PERM_RW_RW); /* CS1 EIM control*/
       X_ARM_MMU_SECTION(0xCC0, 0xCC0, 0x040,
                       ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                       ARM_ACCESS_PERM_RW_RW); /* CS4/5/NAND Flash buffer */
}
#endif

int dram_init(void)
{
	/*
	 * TODO
	 * This should be properly initialized once we know how to
	 * determine the amount of SDRAM we have
	 */
	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size = PHYS_SDRAM_1_SIZE;
	return 0;
}

void setup_nfc(void)
{
	int i;

	/*
	 * The nand pins should be already configured if we are booting
	 * from the nand, but may be the bootstrap is different.
	 * Therefore, we reconfig the pins here and the nand controller.
	 * TODO: we should check if we are booting from flash or SD card
	 * and, based on that, reconfigure the nand controller.
	 */
	mxc_request_iomux(MX51_PIN_NANDF_D0, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_D1, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_D2, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_D3, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_D4, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_D5, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_D6, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_D7, IOMUX_CONFIG_ALT0);

	mxc_request_iomux(MX51_PIN_NANDF_CS0, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_RE_B, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_WE_B, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_WP_B, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_ALE, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_CLE, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_NANDF_RB0, IOMUX_CONFIG_ALT0);

	/* initial NAND setup */

	/* Unlock CS and block addresses */
	for (i = 0; i < 8; i++) {
		writel(0xFFFF0000, REG_UNLOCK_BLK_ADD0 + i * 4);
		writel(0x84 | (i << 3), NFC_WRPROT);
	}

	writel(0, NFC_IPC);

	/*
	 * TODO: when supporting multiple nands with different features
	 * requiering different settings, we should try to read the flash
	 * ID code to ensure we are configuring this correctly
	 */

	/* Configure for 2KB page size */
	writel(0, NFC_CONFIG1);
	writel(0x70200ebd, NFC_CONFIG2);
	writel(0x00100609, NFC_CONFIG3);
}

#define CSPI1_MOSI_PAD_CFG	(PAD_CTL_SRE_FAST | PAD_CTL_DRV_HIGH | \
				PAD_CTL_HYS_ENABLE)
#define CSPI1_MISO_PAD_CFG	(PAD_CTL_SRE_FAST | PAD_CTL_DRV_HIGH | \
				PAD_CTL_HYS_ENABLE)
#define CSPI1_SSX_PAD_CFG_UP	(PAD_CTL_SRE_FAST | PAD_CTL_DRV_HIGH | \
				PAD_CTL_PKE_ENABLE | PAD_CTL_HYS_ENABLE)
#define CSPI1_SSX_PAD_CFG_DOWN	(PAD_CTL_SRE_FAST | PAD_CTL_DRV_HIGH | \
				PAD_CTL_PKE_ENABLE)
#define CSPI1_RDY_PAD_CFG	(PAD_CTL_PKE_ENABLE | PAD_CTL_HYS_ENABLE)
#define CSPI1_SCLK_PAD_CFG	(PAD_CTL_SRE_FAST | PAD_CTL_DRV_HIGH | \
				PAD_CTL_HYS_ENABLE)

void spi_io_init(struct imx_spi_dev_t *dev)
{
	switch (dev->base) {
	case CSPI1_BASE_ADDR:
		/* 000: Select mux mode: ALT0 mux port: MOSI of instance: ecspi1 */
		mxc_request_iomux(MX51_PIN_CSPI1_MOSI, IOMUX_CONFIG_ALT0);
		mxc_iomux_set_pad(MX51_PIN_CSPI1_MOSI, CSPI1_MOSI_PAD_CFG);

		/* 000: Select mux mode: ALT0 mux port: MISO of instance: ecspi1. */
		mxc_request_iomux(MX51_PIN_CSPI1_MISO, IOMUX_CONFIG_ALT0);
		mxc_iomux_set_pad(MX51_PIN_CSPI1_MISO, CSPI1_MISO_PAD_CFG);

		if (dev->ss == 0) {
			/* de-select SS1 of instance: ecspi1. */
			mxc_request_iomux(MX51_PIN_CSPI1_SS1, IOMUX_CONFIG_ALT3);
			mxc_iomux_set_pad(MX51_PIN_CSPI1_SS1, CSPI1_SSX_PAD_CFG_DOWN);
			/* 000: Select mux mode: ALT0 mux port: SS0 of instance: ecspi1. */
			mxc_request_iomux(MX51_PIN_CSPI1_SS0, IOMUX_CONFIG_ALT0);
			mxc_iomux_set_pad(MX51_PIN_CSPI1_SS0, CSPI1_SSX_PAD_CFG_UP);
		} else if (dev->ss == 1) {
			/* de-select SS0 of instance: ecspi1. */
			mxc_request_iomux(MX51_PIN_CSPI1_SS0, IOMUX_CONFIG_ALT3);
			mxc_iomux_set_pad(MX51_PIN_CSPI1_SS0, CSPI1_SSX_PAD_CFG_DOWN);
			/* 000: Select mux mode: ALT0 mux port: SS1 of instance: ecspi1. */
			mxc_request_iomux(MX51_PIN_CSPI1_SS1, IOMUX_CONFIG_ALT0);
			mxc_iomux_set_pad(MX51_PIN_CSPI1_SS1, CSPI1_SSX_PAD_CFG_UP);
		}

		/* 000: Select mux mode: ALT0 mux port: RDY of instance: ecspi1. */
		mxc_request_iomux(MX51_PIN_CSPI1_RDY, IOMUX_CONFIG_ALT0);
		mxc_iomux_set_pad(MX51_PIN_CSPI1_RDY, CSPI1_RDY_PAD_CFG);

		/* 000: Select mux mode: ALT0 mux port: SCLK of instance: ecspi1. */
		mxc_request_iomux(MX51_PIN_CSPI1_SCLK, IOMUX_CONFIG_ALT0);
		mxc_iomux_set_pad(MX51_PIN_CSPI1_SCLK, CSPI1_SCLK_PAD_CFG);
		break;
	case CSPI2_BASE_ADDR:
	default:
		break;
	}
}

#ifdef CONFIG_USER_KEY
#define USER_KEY_PAD_CFG	(PAD_CTL_SRE_SLOW | PAD_CTL_DRV_MEDIUM | \
				 PAD_CTL_ODE_OPENDRAIN_NONE | PAD_CTL_HYS_ENABLE | \
				 PAD_CTL_PUE_KEEPER | PAD_CTL_PKE_NONE | PAD_CTL_DRV_VOT_LOW)
static void setup_user_keys(void)
{
	/*
	 * Configure iomux for gpio operation and the corresponding
	 * gpios as inputs
	 */
	mxc_request_iomux(USER_KEY1_GPIO, USER_KEY1_MUX_MODE);
	mxc_iomux_set_pad(USER_KEY1_GPIO, USER_KEY_PAD_CFG);
	mxc_request_iomux(USER_KEY2_GPIO, USER_KEY2_MUX_MODE);
	mxc_iomux_set_pad(USER_KEY2_GPIO, USER_KEY_PAD_CFG);

	/* Confgiure direction as input */
	imx_gpio_pin_cfg_dir(USER_KEY1_GPIO, 0);
	imx_gpio_pin_cfg_dir(USER_KEY2_GPIO, 0);
}

static void run_user_keys(void)
{
	char cmd[60];

	if (imx_gpio_get_pin(USER_KEY1_GPIO) == 0) {
		printf("\nUser Key 1 pressed\n");
		if (getenv("key1") != NULL) {
			sprintf(cmd, "run key1");
			run_command(cmd, 0);
		}
	}

	if (imx_gpio_get_pin(USER_KEY2_GPIO) == 0) {
		printf("\nUser Key 2 pressed\n");
		if (getenv("key2") != NULL) {
			sprintf(cmd, "run key2");
			run_command(cmd, 0);
		}
	}
}
#endif

#ifdef CONFIG_USER_LED
static void setup_user_leds(void)
{
#define USER_LED_PAD_CFG	(PAD_CTL_SRE_SLOW | PAD_CTL_DRV_MEDIUM | \
				 PAD_CTL_ODE_OPENDRAIN_ENABLE | PAD_CTL_HYS_NONE | \
				 PAD_CTL_PUE_KEEPER | PAD_CTL_PKE_NONE | PAD_CTL_DRV_VOT_LOW)
	/*
	 * Configure iomux for gpio operation and the corresponding
	 * gpios as inputs
	 */
	mxc_request_iomux(USER_LED1_GPIO, USER_LED1_MUX_MODE);
	mxc_iomux_set_pad(USER_LED1_GPIO, USER_LED_PAD_CFG);
	mxc_request_iomux(USER_LED2_GPIO, USER_LED2_MUX_MODE);
	mxc_iomux_set_pad(USER_LED2_GPIO, USER_LED_PAD_CFG);

	/* Confgiure direction as output */
	imx_gpio_pin_cfg_dir(USER_LED1_GPIO, 1);
	imx_gpio_pin_cfg_dir(USER_LED2_GPIO, 1);
}
#endif

#ifdef CONFIG_I2C_MXC
#define I2C_PAD_CFG	(PAD_CTL_SRE_FAST | PAD_CTL_DRV_HIGH | \
	PAD_CTL_ODE_OPENDRAIN_ENABLE | PAD_CTL_100K_PU | PAD_CTL_HYS_ENABLE)
static void setup_i2c(unsigned int module_base)
{
	switch (module_base) {
	case I2C1_BASE_ADDR:
		/* SCL */
		mxc_request_iomux(MX51_PIN_SD2_CMD, IOMUX_CONFIG_ALT1 | IOMUX_CONFIG_SION);
		mxc_iomux_set_pad(MX51_PIN_SD2_CMD, I2C_PAD_CFG);
		mxc_iomux_set_input(MUX_IN_I2C1_IPP_SCL_IN_SELECT_INPUT, INPUT_CTL_PATH2);
		/* SDA */
		mxc_request_iomux(MX51_PIN_SD2_CLK, IOMUX_CONFIG_ALT1 | IOMUX_CONFIG_SION);
		mxc_iomux_set_pad(MX51_PIN_SD2_CLK, I2C_PAD_CFG);
		mxc_iomux_set_input(MUX_IN_I2C1_IPP_SDA_IN_SELECT_INPUT, INPUT_CTL_PATH2);
		break;

	case I2C2_BASE_ADDR:
		/* SCL */
		mxc_request_iomux(MX51_PIN_GPIO1_2, IOMUX_CONFIG_ALT2 | IOMUX_CONFIG_SION);
		mxc_iomux_set_pad(MX51_PIN_GPIO1_2, I2C_PAD_CFG);
		mxc_iomux_set_input(MUX_IN_I2C2_IPP_SCL_IN_SELECT_INPUT, INPUT_CTL_PATH3);
		/* SDA */
		mxc_request_iomux(MX51_PIN_GPIO1_3, IOMUX_CONFIG_ALT2 | IOMUX_CONFIG_SION);
		mxc_iomux_set_pad(MX51_PIN_GPIO1_3, I2C_PAD_CFG);
		mxc_iomux_set_input(MUX_IN_I2C2_IPP_SDA_IN_SELECT_INPUT, INPUT_CTL_PATH3);
		break;

	default:
		printf("Invalid I2C base: 0x%x\n", module_base);
		break;
	}
}
#endif

static void setup_fec(void)
{
#ifdef CONFIG_MXC_FEC
#define CCWMX51_FEC_PAD		(PAD_CTL_DRV_MEDIUM | PAD_CTL_PUE_KEEPER | \
				 PAD_CTL_HYS_ENABLE | PAD_CTL_DRV_VOT_HIGH)

	/* The CCWMX51 module maps the FEC pins to DISP2/DI2 interface */

	/* FEC COL muxed as ALT2 of DISP_DATA10 */
	mxc_request_iomux(MX51_PIN_DISP2_DAT10, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DISP2_DAT10, CCWMX51_FEC_PAD);

	/* FEC CRS muxed as ALT2 of DI2_PIN4 */
	mxc_request_iomux(MX51_PIN_DI2_PIN4, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DI2_PIN4, CCWMX51_FEC_PAD);

	/* FEC MDC muxed as ALT2 of DI2_PIN2 */
	mxc_request_iomux(MX51_PIN_DI2_PIN2, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DI2_PIN2, CCWMX51_FEC_PAD);

	/* FEC MDIO muxed as ALT2 of DI2_PIN3 */
	mxc_request_iomux(MX51_PIN_DI2_PIN3, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DI2_PIN3, CCWMX51_FEC_PAD);

	/* FEC RDATA[0] muxed as ALT2 of DISP2_DAT14 */
	mxc_request_iomux(MX51_PIN_DISP2_DAT14, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DISP2_DAT14, CCWMX51_FEC_PAD);

	/* FEC RDATA[1] muxed as ALT2 of DI2_DISP_CLK */
	mxc_request_iomux(MX51_PIN_DI2_DISP_CLK, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DI2_DISP_CLK, CCWMX51_FEC_PAD);

	/* FEC RDATA[2] muxed as ALT2 of DI_GP4 */
	mxc_request_iomux(MX51_PIN_DI_GP4, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DI_GP4, CCWMX51_FEC_PAD);

	/* FEC RDATA[3] muxed as ALT2 of DISP2_DAT0 */
	mxc_request_iomux(MX51_PIN_DISP2_DAT0, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DISP2_DAT0, CCWMX51_FEC_PAD);

	/* FEC RX_CLK muxed as ALT2 of DISP2_DAT11 */
	mxc_request_iomux(MX51_PIN_DISP2_DAT11, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DISP2_DAT11, CCWMX51_FEC_PAD);

	/* FEC RX_DV muxed as ALT2 of DISP2_DAT12 */
	mxc_request_iomux(MX51_PIN_DISP2_DAT12, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DISP2_DAT12, CCWMX51_FEC_PAD);

	/* FEC RX_ER muxed as ALT2 of DISP2_DAT1 */
	mxc_request_iomux(MX51_PIN_DISP2_DAT1, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DISP2_DAT1, CCWMX51_FEC_PAD);

	/* FEC TDATA[0] muxed as ALT2 of DISP2_DAT15 */
	mxc_request_iomux(MX51_PIN_DISP2_DAT15, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DISP2_DAT15, CCWMX51_FEC_PAD);

	/* FEC TDATA[1] muxed as ALT2 of DISP2_DAT6 */
	mxc_request_iomux(MX51_PIN_DISP2_DAT6, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DISP2_DAT6, CCWMX51_FEC_PAD);

	/* FEC TDATA[2] muxed as ALT2 of DISP2_DAT7 */
	mxc_request_iomux(MX51_PIN_DISP2_DAT7, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DISP2_DAT7, CCWMX51_FEC_PAD);

	/* FEC TDATA[3] muxed as ALT2 of DISP2_DAT8 */
	mxc_request_iomux(MX51_PIN_DISP2_DAT8, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DISP2_DAT8, CCWMX51_FEC_PAD);

	/* FEC TX_CLK muxed as ALT2 of DISP2_DAT13 */
	mxc_request_iomux(MX51_PIN_DISP2_DAT13, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DISP2_DAT13, CCWMX51_FEC_PAD);

	/* FEC TX_EN muxed as ALT2 of DISP2_DAT9 */
	mxc_request_iomux(MX51_PIN_DISP2_DAT9, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DISP2_DAT9, CCWMX51_FEC_PAD);

	/* FEC TX_ER muxed as ALT2 of DI_GP3 */
	mxc_request_iomux(MX51_PIN_DI_GP3, IOMUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX51_PIN_DI_GP3, CCWMX51_FEC_PAD);

	/* FEC signals involved in daisy chain should be mapped to DISP2/DI2 pads */
	mxc_iomux_set_input(MUX_IN_FEC_FEC_COL_SELECT_INPUT, INPUT_CTL_PATH1);
	mxc_iomux_set_input(MUX_IN_FEC_FEC_CRS_SELECT_INPUT, INPUT_CTL_PATH1);
	mxc_iomux_set_input(MUX_IN_FEC_FEC_MDI_SELECT_INPUT, INPUT_CTL_PATH1);
	mxc_iomux_set_input(MUX_IN_FEC_FEC_RDATA_0_SELECT_INPUT, INPUT_CTL_PATH1);
	mxc_iomux_set_input(MUX_IN_FEC_FEC_RDATA_1_SELECT_INPUT, INPUT_CTL_PATH1);
	mxc_iomux_set_input(MUX_IN_FEC_FEC_RDATA_2_SELECT_INPUT, INPUT_CTL_PATH1);
	mxc_iomux_set_input(MUX_IN_FEC_FEC_RDATA_3_SELECT_INPUT, INPUT_CTL_PATH1);
	mxc_iomux_set_input(MUX_IN_FEC_FEC_RX_CLK_SELECT_INPUT, INPUT_CTL_PATH1);
	mxc_iomux_set_input(MUX_IN_FEC_FEC_RX_DV_SELECT_INPUT, INPUT_CTL_PATH1);
	mxc_iomux_set_input(MUX_IN_FEC_FEC_RX_ER_SELECT_INPUT, INPUT_CTL_PATH1);
	mxc_iomux_set_input(MUX_IN_FEC_FEC_TX_CLK_SELECT_INPUT, INPUT_CTL_PATH1);

	/*
	 * TODO: If clock were inactive due to clock gatting, enable it now
	 * Revisit when power scheme is clearly defined.
	 */
#endif
}



static void power_init(void)
{
	unsigned int val;
#define REV_ATLAS_LITE_1_0	   0x8
#define REV_ATLAS_LITE_1_1	   0x9
#define REV_ATLAS_LITE_2_0	   0x10
#define REV_ATLAS_LITE_2_1	   0x11

	if (is_soc_rev(CHIP_REV_2_0) >= 0) {
		/* Set core voltage to 1.1V */
		val = pmic_reg(pmicslv, 24, 0, 0);
		val = (val & (~0x1F)) | 0x14;
		pmic_reg(pmicslv, 24, val, 1);

		/* Setup VCC (SW2) to 1.25 */
		val = pmic_reg(pmicslv, 25, 0, 0);
		val = (val & (~0x1F)) | 0x1A;
		pmic_reg(pmicslv, 25, val, 1);

		/* Setup 1V2_DIG1 (SW3) to 1.2 */
		val = pmic_reg(pmicslv, 26, 0, 0);
		val = (val & (~0x1F)) | 0x18;
		pmic_reg(pmicslv, 25, val, 1);

		udelay(50);
		/* Press the turbo... and increase freq to 800MHz */
		writel(0x0, CCM_BASE_ADDR + CLKCTL_CACRR);
	} else {
		printf("*** WARNING: please define power settings for TO3\n");
		while (1) ;
	}

	if (((pmic_reg(pmicslv, 7, 0, 0) & 0x1F) < REV_ATLAS_LITE_2_0) ||
		(((pmic_reg(pmicslv, 7, 0, 0) >> 9) & 0x3) == 0)) {
		/* Set switchers in PWM mode for Atlas 2.0 and lower */
		/* Setup the switcher mode for SW1 & SW2*/
		val = pmic_reg(pmicslv, 28, 0, 0);
		val = (val & (~0x3C0F)) | 0x1405;
		pmic_reg(pmicslv, 28, val, 1);

		/* Setup the switcher mode for SW3 & SW4 */
		val = pmic_reg(pmicslv, 29, 0, 0);
		val = (val & (~0xF0F)) | 0x505;
		pmic_reg(pmicslv, 29, val, 1);
	} else {
		/* Set switchers in Auto in NORMAL mode & STANDBY mode for Atlas 2.0a */
		/* Setup the switcher mode for SW1 & SW2*/
		val = pmic_reg(pmicslv, 28, 0, 0);
		val = (val & (~0x3C0F)) | 0x2008;
		pmic_reg(pmicslv, 28, val, 1);

		/* Setup the switcher mode for SW3 & SW4 */
		val = pmic_reg(pmicslv, 29, 0, 0);
		val = (val & (~0xF0F)) | 0x808;
		pmic_reg(pmicslv, 29, val, 1);
	}

	/* Set VVIDEO to 2.775V, VAUDIO to 3V, VSD to 3.15V */
	val = pmic_reg(pmicslv, 31, 0, 0);
	val &= ~0x1FC;
	val |= 0x1F4;
	pmic_reg(pmicslv, 31, val, 1);

	/* Configure VGEN3 and VCAM regulators to use external PNP */
	val = 0x208;
	pmic_reg(pmicslv, 33, val, 1);
	udelay(200);

	/* Enable VGEN3, VCAM, VAUDIO, VVIDEO, VSD regulators */
	val = 0x49249;
	pmic_reg(pmicslv, 33, val, 1);

	/* Set all PMIC GPOs to low */
	val = pmic_reg(pmicslv, 34, 0, 0);
	val &= ~PMIC_SET_ALL_GPO_LOW_MSK;
	val = pmic_reg(pmicslv, 34, val, 1);

	/* Enable USB OTG regulator */
	val = 0x409;
	pmic_reg(pmicslv, 50, val, 1);
	udelay(100);

	/* Reset the external devices by clearing GPO2, GPO3 and GPO4 */
	val = pmic_reg(pmicslv, 34, val, 0);
	val &= ~((3 << 12) | (3 << 10) | (3 << 8));
	val = pmic_reg(pmicslv, 34, val, 1);
	udelay(100);
	val |= (1 << 12) | (1 << 10) | (1 << 8);
	val = pmic_reg(pmicslv, 34, val, 1);
	udelay(100);
}

#ifdef CONFIG_NET_MULTI
/* TODO, this shouldnt depend on net_multi. Maybe we only have this interface enabled? */
#if defined(CONFIG_SMC911X)
#include <net.h>
#include "../../../drivers/net/smc911x.h"
u32 mx51_io_base_addr;
extern int smc911x_initialize(u8 dev_num, int base_addr);
#endif
int board_eth_init(bd_t *bis)
{
	int rc = -ENODEV;
#if defined(CONFIG_SMC911X)
	struct eth_device *dev;
	nv_critical_t *pNvram;
	unsigned long addrh, addrl;
	uchar *m;

	/* Configure the pin mux for CS line and address and data buses */
	mxc_request_iomux(MX51_PIN_EIM_CS5, IOMUX_CONFIG_ALT0);
	mxc_iomux_set_pad(MX51_PIN_EIM_CS5, PAD_CTL_HYS_ENABLE | PAD_CTL_PUE_KEEPER | PAD_CTL_DRV_MEDIUM);
	mxc_request_iomux(MX51_PIN_EIM_OE, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_DA0, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_DA1, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_DA2, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_DA3, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_DA4, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_DA5, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_DA6, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_DA7, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D16, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D17, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D18, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D19, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D20, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D21, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D22, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D23, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D24, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D25, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D26, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D27, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D28, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D29, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D30, IOMUX_CONFIG_ALT0);
	mxc_request_iomux(MX51_PIN_EIM_D31, IOMUX_CONFIG_ALT0);

	/* Configure the CS timming, bus width, etc */
	/* 16 bit on DATA[31..16], not multiplexed, async */
	writel(0x00420081, WEIM_BASE_ADDR + 0x78 + CSGCR1);
	/* ADH has not effect on non muxed bus */
	writel(0, WEIM_BASE_ADDR + 0x78 + CSGCR2);
	/* RWSC=50, RADVA=2, RADVN=6, OEA=0, OEN=0, RCSA=0, RCSN=0 */
	writel(0x32260000, WEIM_BASE_ADDR + 0x78+ CSRCR1);
	/* APR=0 */
	writel(0, WEIM_BASE_ADDR + 0x78 + CSRCR2);
	/* WAL=0, WBED=1, WWSC=50, WADVA=2, WADVN=6, WEA=0, WEN=0, WCSA=0 */
	writel(0x72080f00, WEIM_BASE_ADDR + 0x78 + CSWCR1);

	mx51_io_base_addr = CS5_BASE_ADDR;

	/* ...and lets initialize the device */
	rc = smc911x_initialize(0, CONFIG_SMC911X_BASE);
	if (!rc) {
		/* set mac address even if there is no access to the network */  
		if ((dev = eth_get_dev_by_name("smc911x-0")) != NULL) {
		  	if (!NvCriticalGet(&pNvram))
				return -1;
			m = &pNvram->s.p.eth1addr;

			addrl = m[0] | (m[1] << 8) | (m[2] << 16) | (m[3] << 24);
			addrh = m[4] | (m[5] << 8);
			smc911x_set_mac_csr(dev, ADDRL, addrl);
			smc911x_set_mac_csr(dev, ADDRH, addrh);
		}
	}
#endif
	return rc;
}
#endif

#ifdef CONFIG_CMD_MMC
volatile u32 *imx_esdhc_base_addr;

#define SD_PAD_CFG	(PAD_CTL_DRV_VOT_HIGH |	\
			 PAD_CTL_47K_PU |	\
			 PAD_CTL_DRV_HIGH)
int esdhc_gpio_init(int mmc_index)
{
	s32 status = 0;

	switch (mmc_index) {
	case 0:
		imx_esdhc_base_addr = (volatile u32 *)MMC_SDHC1_BASE_ADDR;

		/* SD1_CMD */
		mxc_request_iomux(MX51_PIN_SD1_CMD, IOMUX_CONFIG_ALT0 | IOMUX_CONFIG_SION);
		mxc_iomux_set_pad(MX51_PIN_SD1_CMD, SD_PAD_CFG);

		/* SD1_CLK */
		mxc_request_iomux(MX51_PIN_SD1_CLK, IOMUX_CONFIG_ALT0 | IOMUX_CONFIG_SION);
		mxc_iomux_set_pad(MX51_PIN_SD1_CLK, SD_PAD_CFG);

		/* SD1_DATA0 */
		mxc_request_iomux(MX51_PIN_SD1_DATA0, IOMUX_CONFIG_ALT0 | IOMUX_CONFIG_SION);
		mxc_iomux_set_pad(MX51_PIN_SD1_DATA0, SD_PAD_CFG);

		/* SD1_DATA1 */
		mxc_request_iomux(MX51_PIN_SD1_DATA1, IOMUX_CONFIG_ALT0 | IOMUX_CONFIG_SION);
		mxc_iomux_set_pad(MX51_PIN_SD1_DATA1, SD_PAD_CFG);

		/* SD1_DATA2 */
		mxc_request_iomux(MX51_PIN_SD1_DATA2, IOMUX_CONFIG_ALT0 | IOMUX_CONFIG_SION);
		mxc_iomux_set_pad(MX51_PIN_SD1_DATA2, SD_PAD_CFG);

		/* SD1_DATA3 */
		mxc_request_iomux(MX51_PIN_SD1_DATA3, IOMUX_CONFIG_ALT0 | IOMUX_CONFIG_SION);
		mxc_iomux_set_pad(MX51_PIN_SD1_DATA3, SD_PAD_CFG);
		break;

		status = 1;
		break;
	case 2:
		imx_esdhc_base_addr = (volatile u32 *)MMC_SDHC3_BASE_ADDR;

		/* SD3_CMD */
		mxc_request_iomux(MX51_PIN_NANDF_RDY_INT, IOMUX_CONFIG_ALT5 | IOMUX_CONFIG_SION);
		mxc_iomux_set_pad(MX51_PIN_NANDF_RDY_INT, SD_PAD_CFG);

		/* SD3_CLK */
		mxc_request_iomux(MX51_PIN_NANDF_CS7, IOMUX_CONFIG_ALT5);
		mxc_iomux_set_pad(MX51_PIN_NANDF_CS7, SD_PAD_CFG);

		/* SD3_DATA0 */
		mxc_request_iomux(MX51_PIN_NANDF_D8, IOMUX_CONFIG_ALT5);
		mxc_iomux_set_pad(MX51_PIN_NANDF_D8, SD_PAD_CFG);
		mxc_iomux_set_input(MUX_IN_ESDHC3_IPP_DAT0_IN_SELECT_INPUT, INPUT_CTL_PATH1);

		/* SD3_DATA1 */
		mxc_request_iomux(MX51_PIN_NANDF_D9, IOMUX_CONFIG_ALT5);
		mxc_iomux_set_pad(MX51_PIN_NANDF_D9, SD_PAD_CFG);
		mxc_iomux_set_input(MUX_IN_ESDHC3_IPP_DAT1_IN_SELECT_INPUT, INPUT_CTL_PATH1);

		/* SD3_DATA2 */
		mxc_request_iomux(MX51_PIN_NANDF_D10, IOMUX_CONFIG_ALT5);
		mxc_iomux_set_pad(MX51_PIN_NANDF_D10, SD_PAD_CFG);
		mxc_iomux_set_input(MUX_IN_ESDHC3_IPP_DAT2_IN_SELECT_INPUT, INPUT_CTL_PATH1);

		/* SD3_DATA3 */
		mxc_request_iomux(MX51_PIN_NANDF_D11, IOMUX_CONFIG_ALT5);
		mxc_iomux_set_pad(MX51_PIN_NANDF_D11, SD_PAD_CFG);
		mxc_iomux_set_input(MUX_IN_ESDHC3_IPP_DAT3_IN_SELECT_INPUT, INPUT_CTL_PATH1);
		break;
	case 1:
	case 3:
	default:
		printf("*** ERROR: SD/MMC interface %d not supported\n", mmc_index);
		status = 1;
		break;
	}

	return status;
}

int board_mmc_init(void)
{
	int mmc_devs[] = MMC_DEVICES;
	int i;

	for(i=0; i < ARRAY_SIZE(mmc_devs); i++) {
		if (!esdhc_gpio_init(mmc_devs[i]))
			fsl_esdhc_mmc_init(gd->bd);
		else
			return -1;
	}
	return 0;
}
#endif

int board_init(void)
{
#ifndef CONFIG_ICACHE_L1_OFF
	icache_enable();
#endif
	setup_soc_rev();

	gd->bd->bi_arch_number = MACH_TYPE_CCWMX51JS;
	/* address of boot parameters */
	gd->bd->bi_boot_params = PHYS_SDRAM_1 + 0x100;

#ifdef CONFIG_USER_LED
	setup_user_leds();
#endif
	setup_nfc();
	setup_fec();
#ifdef CONFIG_I2C_MXC
	setup_i2c(CONFIG_SYS_I2C_PORT);
	i2c_init(CONFIG_SYS_I2C_SPEED, CONFIG_SYS_I2C_SLAVE);
#endif
#ifdef CONFIG_USER_KEY
	setup_user_keys();
#endif
	return 0;
}

#ifdef CONFIG_PRINT_SYS_INFO
extern void mxc_dump_clocks(void);
extern u32 __get_mcu_main_clk(void);
extern void show_pmic_info(struct spi_slave *slave);

void print_sys_info(void)
{
	char *s;

	if ((s = getenv("bv")) != NULL)
		boot_verb = simple_strtoul(s, NULL, 10);

	if (boot_verb) {

		printf("CPU:   Freescale i.MX51 family %d.%dV at %d MHz\n",
				(get_board_rev() & 0xFF) >> 4,
				(get_board_rev() & 0xF),
				__get_mcu_main_clk() / 1000000);
		show_pmic_info(pmicslv);
		mxc_dump_clocks();

		printf("Last reset was: ");

		switch (__REG(SRC_BASE_ADDR + 0x8)) {
		case 0x0001:	printf("POR\n");	break;
		case 0x0009:	printf("RST\n");	break;
		case 0x0010:
		case 0x0011:	printf("WDOG\n");	break;
		default:	printf("unknown\n");
		}
	}
}
#endif

#ifdef BOARD_LATE_INIT
int board_late_init(void)
{
	nv_critical_t *pNvram;

	/* Switch on voltage regulators and reset some peripherals */
	if (!spi_pmic_init(0)) {
		printf("***ERROR: unable to setup SPI for PMIC\n");
		return -1;
	}

	power_init();

	if (NvCriticalGet(&pNvram)) {
		/* Pass location of environment in memory to OS through bi */
		gd->bd->nvram_addr = (unsigned long)pNvram;
	}
	memcpy((int*)gd->bd->bi_boot_params, gd->bd, sizeof(bd_t));

	return 0;
}
#endif

#ifdef BOARD_BEFORE_MLOOP_INIT
int board_before_mloop_init (void)
{
#ifdef CONFIG_PRINT_SYS_INFO
	print_sys_info();
#endif
#ifdef CONFIG_USER_KEY
	run_user_keys();
#endif
	return 0;
}
#endif

int checkboard(void)
{
	printf("Board: CCWMX51 JSK Development Board\n");
	return 0;
}

#ifdef CONFIG_CMD_PMIC
int do_pmic_r(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	ulong reg, count = 1, i;

	if ((argc < 2) || (argc > 3))
		goto usage;

	if (argc == 3)
		count = simple_strtoul(argv[2], NULL, 10);

	reg = simple_strtoul(argv[1], NULL, 10);

	if ((reg + count - 1) > 63) {
		printf ("*** ERROR: invalid parameters, last addr should be less than 64\n");
		return 1;
	}

	if (!pmicslv) {
		printf ("*** ERROR: PMIC device not initialized yet\n");
		return 1;
	}

	for (i = reg; i < (reg + count); i++) {
		if (!((i - reg) % 4))
			printf("%s%02d:", (i - reg) ? "\n" : "", (int)i);
		printf(" %08x", pmic_reg(pmicslv, i, 0, 0));
	}
	printf("\n");
	return 0;

usage:
	printf ("Usage:\n%s\n", cmdtp->usage);
	return 1;
}

int do_pmic_w(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	ulong reg, val;

	if (argc != 3)
		goto usage;

	reg = simple_strtoul(argv[1], NULL, 10);
	val = simple_strtoul(argv[2], NULL, 16);

	if (reg > 63) {
		printf ("*** ERROR: invalid parameters, reg addr should be less than 64\n");
		return 1;
	}

	if (!pmicslv) {
		printf ("*** ERROR: PMIC device not initialized yet\n");
		return 1;
	}

	pmic_reg(pmicslv, reg, val, 1);
	return 0;

usage:
	printf ("Usage:\n%s\n", cmdtp->usage);
	return 1;
}

int do_pmic(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	if (strlen(argv[0]) != 6 || argv[0][4] != '.' ||
	   (argv[0][5] != 'r' && argv[0][5] != 'w'))
		goto error;
	if (argv[0][5] == 'r')
		return do_pmic_r(cmdtp, flag, argc, argv);
	if (argv[0][5] == 'w')
		return do_pmic_w(cmdtp, flag, argc, argv);
error:
	printf ("Usage:\n%s\n", cmdtp->usage);
	return 1;
}

U_BOOT_CMD(
	pmic,	3,	1,	do_pmic,
	"pmic register access",
	".r address count    - Read count registers starting from address (values in dec)\n"
	"     .w address value    - Write register at address (in dec) with value (in hex)\n"
);
#endif /* CONFIG_CMD_PMIC */

