/*
 * Copyright (C) 2007, Guennadi Liakhovetski <lg@denx.de>
 *
 * (C) Copyright 2009 Freescale Semiconductor, Inc.
 *
 * Configuration settings for the MX51-3Stack Freescale board.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H
#include "digi_common.h"		/* Load Digi common stuff... */
#include <asm/arch/mx51.h>

 /* High Level Configuration Options */
#define CONFIG_ARMV7			1	/* This is armv7 Cortex-A8 CPU core */
#define CONFIG_SYS_APCS_GNU

#define CONFIG_MXC			1
#define CONFIG_CCWMX51		1	/* in a CCWi-MX51 */
#define CONFIG_ARCH_CPU_INIT
#define CONFIG_ARCH_MMU

#ifndef CONFIG_DOWNLOAD_BY_DEBUGGER
# define CONFIG_FLASH_HEADER		1
# define CONFIG_FLASH_HEADER_OFFSET	0x400
# define CONFIG_FLASH_HEADER_BARKER	0xB1
# define CONFIG_CMD_AUTOSCRIPT		1
# define CONFIG_AUTOLOAD_BOOTSCRIPT	1
# define CONFIG_SOURCE			1
#endif

#define CONFIG_SKIP_RELOCATE_UBOOT

#define CONFIG_MX51_HCLK_FREQ		24000000	/* RedBoot says 26MHz */

//#define CONFIG_DISPLAY_CPUINFO
//#define CONFIG_DISPLAY_BOARDINFO

#define BOARD_LATE_INIT
#define	BOARD_BEFORE_MLOOP_INIT

/*
 * Disabled for now due to build problems under Debian and a significant
 * increase in the final file size: 144260 vs. 109536 Bytes.
 */

#define CONFIG_CMDLINE_TAG		1	/* enable passing of ATAGs */
#define CONFIG_REVISION_TAG		1
#define CONFIG_SETUP_MEMORY_TAGS	1
#define CONFIG_INITRD_TAG		1


/**
 * Digi commands and defines
 */
#define PLATFORM			"js"
#define CONFIG_MODULE_NAME		"ccwmx51"PLATFORM
#define	CONFIG_LOAD_ADDR		0x90800000	/* default load address	*/
#define CONFIG_WCE_LOAD_ADDR		0x90200000
#define CONFIG_INITRD_LOAD_ADDR		0x90600000
#define EBOOTFLADDR			"280000"
#define CONFIG_PRINT_SYS_INFO		1
#define	CONFIG_MODULE_NAME_WCE		"CCXMX51"
#define CONFIG_MODULE_STRING		"ConnectCore Wi-i.MX51"
#define CONFIG_UBOOT_BOARDNAME_STR	"a JSK Development Board"
#define CONFIG_IDENT_STRING		" - GCC " __VERSION__ "\nfor " \
					CONFIG_MODULE_STRING " on " CONFIG_UBOOT_BOARDNAME_STR

#ifndef CONFIG_CONSOLE
# define CONFIG_CONSOLE			"ttymxc"
#endif

/**
 * Hardware drivers
 */
#define CONFIG_MX51_UART		1
#define CONFIG_SERIAL_MULTI
#define CONFIG_CONS_INDEX		1

/**
 * SPI Configs
 */
#define CONFIG_CMD_SPI
#define CONFIG_IMX_ECSPI
#define CONFIG_IMX_SPI_PMIC
#define CONFIG_IMX_SPI_PMIC_CS	0
#define CONFIG_IMX_SPI_PMIC_SPEED	20000000
#define CONFIG_IMX_SPI_SS1_SPEED	2500000
#define	IMX_CSPI_VER_2_3	1
#define MAX_SPI_BYTES		(64 * 4)

/**
 * PMIC command
 */
#define CONFIG_CMD_PMIC

/**
 * RTC PMIC
 */
#define CONFIG_RTC_MC13892	1

/**
 * User keys
 */
#define CONFIG_USER_KEY
#define USER_KEY1_GPIO			MX51_PIN_GPIO1_8
#define USER_KEY2_GPIO			MX51_PIN_GPIO1_1
#define USER_KEY1_MUX_MODE		IOMUX_CONFIG_ALT0
#define USER_KEY2_MUX_MODE		IOMUX_CONFIG_ALT1

/**
 * User and status leds
 */
#define CONFIG_USER_LED
#define USER_LED1_GPIO			MX51_PIN_NANDF_RB2
#define USER_LED2_GPIO			MX51_PIN_NANDF_RB1
#define USER_LED1_MUX_MODE		IOMUX_CONFIG_ALT3
#define USER_LED2_MUX_MODE		IOMUX_CONFIG_ALT3
#define CONFIG_STATUS_LED

/**
 * I2C Configs
 */
#define CONFIG_CMD_I2C			1
#define CONFIG_HARD_I2C			1
#define CONFIG_I2C_MXC			1
#define CONFIG_SYS_I2C_PORT		I2C2_BASE_ADDR
#define CONFIG_SYS_I2C_SPEED		100000
#define CONFIG_SYS_I2C_SLAVE		0xfe

/**
 * MMC Configs
 */
#define CONFIG_IMX_MMC		1
#define MMC_DEVICES		{0, 2}	/* 0-SD1, 1-SD2, 2-SD3, 3-SD4 */
#define CONFIG_MULTI_MMC	1
#define CONFIG_MMC              1
#define CONFIG_GENERIC_MMC
#define CONFIG_CMD_MMC
#define CONFIG_DOS_PARTITION	1
#define CONFIG_CMD_FAT		1

/**
 * Eth Configs
 */
#define CONFIG_ETH_PRIME
#define CONFIG_HAS_ETH1		/* Platform has a second NIC */
#define CONFIG_NET_MULTI			1
#define CONFIG_MII
#define CONFIG_DISCOVER_PHY

/* Support for internal Fast Ethernet Controller */
#define CONFIG_MXC_FEC
#define CONFIG_FEC0_IOBASE			FEC_BASE_ADDR
#define CONFIG_FEC0_PINMUX			-1
#define CONFIG_FEC0_PHY_ADDR			7
#define CONFIG_FEC0_MIIBASE 			-1

/* Support for external Ethernet Controller LAN9221 */
//#define CONFIG_SMC911X			1
#define CONFIG_SMC911X_16_BIT		1
#define CONFIG_SMC911X_BASE		mx51_io_base_addr


/***********************************************************
 * Command definition
 ***********************************************************/

#include <config_cmd_default.h>

#define CONFIG_CMD_NAND
#define CONFIG_MXC_NAND
#undef	CONFIG_CMD_FLASH

#undef CONFIG_CMD_IMLS

/**
 * Miscellaneous configurable options
 */
#define CONFIG_SYS_LONGHELP		/* undef to save memory */
#define CONFIG_SYS_PROMPT		"CCWMX51 # "
#define CONFIG_AUTO_COMPLETE

#define CONFIG_SYS_MEMTEST_START	0x98000000	/* memtest works on */
#define CONFIG_SYS_MEMTEST_END		(0x90000000 + PHYS_SDRAM_1_SIZE)	/* This should be dynamic */

#undef	CONFIG_SYS_CLKS_IN_HZ		/* everything, incl board info, in Hz */

#define CONFIG_SYS_LOAD_ADDR		CONFIG_LOAD_ADDR

#define CONFIG_SYS_HZ			1000

#define CONFIG_CMDLINE_EDITING	1


/*-----------------------------------------------------------------------
 * Physical Memory Map
 */
#define CONFIG_NR_DRAM_BANKS		1
#define PHYS_SDRAM_1			CSD0_BASE_ADDR
/* TO1 boards */
/* #define PHYS_SDRAM_1_SIZE	(128 * 1024 * 1024) */
#define PHYS_SDRAM_1_SIZE		(512 * 1024 * 1024)

/*-----------------------------------------------------------------------
 * FLASH and environment organization
 */
#define CONFIG_SYS_NO_FLASH

/*-----------------------------------------------------------------------
 * NAND FLASH driver setup
 */
#define NAND_MAX_CHIPS			8
#define CONFIG_SYS_MAX_NAND_DEVICE	1
#define CONFIG_SYS_NAND_BASE		0x40000000

/**
 * JFFS2 partitions
 */
#undef CONFIG_JFFS2_CMDLINE
#define CONFIG_JFFS2_DEV	"nand0"

#define PART_WINCE_FS_SIZE    	0x0
#define PART_NETOS_KERNEL_SIZE 	0x002C0000
#define PART_ROOTFS_SIZE       	0x01000000
#define PART_WINCE_SIZE        	0x02800000
#define PART_SPLASH_SIZE       	0x00100000
#define PART_NVRAM_SIZE        	0x00040000	/* nvram 256 kb because 2k pagesize nands */
#define PART_UBOOT_SIZE	       	0x000c0000
#define PART_KERNEL_SIZE       	0x00300000
#define PART_ROOTFS_SIZE       	0x01000000

/**
 * Environment
 */
#define	CONFIG_PLATFORM_ENV_SETTINGS		\
		"ethprime=FEC0\0"

#endif				/* __CONFIG_H */
